{-# OPTIONS_GHC -fglasgow-exts #-}

module ProofChecker where {

import Show;
import Term;
import Loop;
import DpTransImpl;
import List;
import ErrorMonad;
import RedPairImpl;
import DepGraph;
import Option;
import Nat;
import SemanticLabeling;
import SubtermCriterionImpl;
import Util;
import Trs;
import DpFramework;
import Edg;

data TRSDisproof a b = Loop ([(Term a b)], (Ctxt a b, [(Term a b, Term a b)]))
  | NotWellFormed;

data DPProof a b c = PisEmpty
  | RedPairProc (RedPair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (DPProof a b c)
  | RedPairUrProc (RedPair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | DepGraphProc
      [(Maybe (DPProof a b c), [(Term (Lab a b) c, Term (Lab a b) c)])]
  | MonoRedPairProc (RedPair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | MonoRedPairUrProc (RedPair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | UnlabProc (DPProof a b c)
  | SemlabProc (Sl_inter a b) (Lab a b -> [Nat] -> b)
      [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | SpscProc (SpL (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (DPProof a b c);

data TRSProof a b c =
  DPTrans [(Term (Shp (Lab a b)) c, Term (Shp (Lab a b)) c)]
    (DPProof (Shp (Lab a b)) b c)
  | RuleRemoval (RedPair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (TRSProof a b c)
  | Unlab [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | Semlab (Sl_inter a b) (Lab a b -> [Nat] -> b)
      [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | RisEmpty;

debug :: forall a. [Char] -> [Char] -> a -> a;
debug i t x = x;

indent :: [[Char]] -> [[Char]];
indent xs = newline : mapa (append [' ', ' ']) xs;

checkTRSDisproof ::
  forall a b.
    (Eq a, Sshow a, Eq b,
      Sshow b) => [Char] ->
                    [(Term a b, Term a b)] ->
                      TRSDisproof a b -> Eithera [[Char]] ();
checkTRSDisproof i r (Loop l) =
  debug i ['L', 'o', 'o', 'p']
    (catcha (checkLoop r l)
      (\ x ->
        errora
          (append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'l', 'o',
               'o', 'p', ' ', 'p', 'r', 'o', 'o', 'f']
            : x)));
checkTRSDisproof i r NotWellFormed =
  debug i
    ['n', 'o', 't', ' ', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd']
    (catcha (checkNotWellFormed r)
      (\ x ->
        errora
          (append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'n', 'o',
               't', '-', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd',
               ' ', 'p', 'r', 'o', 'o', 'f']
            : x)));

checkDPProof ::
  forall a b c.
    (Eq a, Sshow a, Eq b, Sshow b, Eq c,
      Sshow c) => [Char] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      [(Term (Lab a b) c, Term (Lab a b) c)] ->
                        Maybe (Lab a b ->
                                [(Term (Lab a b) c, Term (Lab a b) c)])
                          -> Maybe (Lab a b ->
                                     [(Term (Lab a b) c, Term (Lab a b) c)])
                               -> DPProof a b c -> Eithera [[Char]] ();
checkDPProof i p r rm rrm PisEmpty =
  debug i ['P', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (case p of {
      [] -> returna ();
      x : xa ->
        errora
          (append i
             [':', ' ', 'P', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
               'p', 't', 'y', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 'f', 'o',
               'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'D', 'P', '-', 'p', 'r',
               'o', 'b', 'l', 'e', 'm', ' ']
            : showDPP p r);
    });
checkDPProof i p r rm rrm (RedPairProc redp rP prf) =
  debug i ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (bind (catcha (createRedPairProc redp p rP r)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ',
                     'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's',
                     'o', 'r', ' ', 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h',
                     ' ', 'f', 'r', 'o', 'm', ' ']
                  : append (showDPP p r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p',
                         'r', 'o', 'b', 'l', 'e', 'm', ' ']
                        : append (showDPP rP r) x))))
      (\ x ->
        catcha (checkDPProof (append i ['.', '1']) rP r rm rrm prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't',
                   'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                   'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (RedPairUrProc redp rP ur prf) =
  debug i
    ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c', ' ', '(', 'u', 's',
      'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ' ', '+', ' ', 'a', 'f',
      ')']
    (bind (catcha (createRedPairUrAfProc redp p rP r rm ur)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ',
                     'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's',
                     'o', 'r', ' ', '(', 'w', 'i', 't', 'h', ' ', 'u', 's', 'a',
                     'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ' ', 'a', 'n',
                     'd', ' ', 'a', 'f', ')', ' ', 't', 'o', ' ', 's', 'w', 'i',
                     't', 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
                  : append (showDPP p r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p',
                         'r', 'o', 'b', 'l', 'e', 'm', ' ']
                        : append (showDPP rP r) x))))
      (\ x ->
        catcha (checkDPProof (append i ['.', '1']) rP r rm rrm prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't',
                   'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                   'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (DepGraphProc edpts) =
  debug i ['D', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
    (bind (catcha
            (checkDepGraphProc p
              (mapa (\ x ->
                      ((case fst x of {
                         Nothing -> False;
                         Just xa -> True;
                       }),
                        snd x))
                edpts)
              rm rrm)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'i', 'l',
                     'e', ' ', 't', 'r', 'y', 'i', 'n', 'g', ' ', 't', 'o', ' ',
                     'p', 'e', 'r', 'f', 'o', 'r', 'm', ' ', 'S', 'C', 'C', '-',
                     'd', 'e', 'c', 'o', 'm', 'p', 'o', 's', 'i', 't', 'i', 'o',
                     'n', ' ', ' ', 'o', 'n', ' ']
                  : append (showDPP p r) x)))
      (\ x ->
        catcha
          (checkAllMIndex
            (\ j rp -> (case fst rp of {
                         Nothing -> j;
                         Just prf -> Suc j;
                       }))
            (\ j rp ->
              (case fst rp of {
                Nothing -> returna ();
                Just a ->
                  checkDPProof (append (append i ['.']) (sshow_nat j)) (snd rp)
                    r rm rrm a;
              }))
            edpts)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'd', 'e', 'p', 'e', 'n', 'd',
                   'e', 'n', 'c', 'y', ' ', 'g', 'r', 'a', 'p', 'h', ' ', 'p',
                   'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (MonoRedPairProc redp rP rR prf) =
  debug i
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (bind (catcha (createMonoRedPairProc redp p rP r rR)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ',
                     'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a',
                     'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r',
                     ' ', 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 'f',
                     'r', 'o', 'm', ' ']
                  : append (showDPP p r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p',
                         'r', 'o', 'b', 'l', 'e', 'm', ' ']
                        : append (showDPP rP rR) x))))
      (\ x ->
        catcha
          (checkDPProof (append i ['.', '1']) rP rR (computeRuleMap rR)
            (computeRuleMap (reverse_trs rR)) prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o',
                   'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                   'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e',
                   's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (MonoRedPairUrProc redp rP rR ur prf) =
  debug i
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c',
      ' ', '(', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ')']
    (bind (catcha (createMonoRedPairUrProc redp p rP r rm rR ur)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ',
                     'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a',
                     'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r',
                     ' ', '(', 'w', 'i', 't', 'h', ' ', 'u', 's', 'a', 'b', 'l',
                     'e', ' ', 'r', 'u', 'l', 'e', 's', ')', ' ', 't', 'o', ' ',
                     's', 'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
                  : append (showDPP p r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p',
                         'r', 'o', 'b', 'l', 'e', 'm', ' ']
                        : append (showDPP rP rR) x))))
      (\ x ->
        catcha
          (checkDPProof (append i ['.', '1']) rP rR (computeRuleMap rR)
            (computeRuleMap (reverse_trs rR)) prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o',
                   'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                   'n', ' ', 'p', 'a', 'i', 'r', ' ', '(', 'w', 'i', 't', 'h',
                   ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e',
                   's', ')', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (UnlabProc prf) =
  debug i ['U', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (catcha
      (checkDPProof (append i ['.', '1']) (mapa (map_rule unlab) p)
        (mapa (map_rule unlab) r) (computeRuleMap (mapa (map_rule unlab) r))
        (computeRuleMap (reverse_trs (mapa (map_rule unlab) r))) prf)
      (\ x ->
        errora
          (append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
               ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a', 'b', 'e', 'l', 'i',
               'n', 'g', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
            : indent x)));
checkDPProof i p r rm rrm (SemlabProc sli l lP lR prf) =
  debug i ['S', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (bind (catcha (create_sem_lab_proc sli l p r lP lR)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 's', 'e', 'm', 'a', 'n', 't', 'i', 'c', ' ', 'l',
                     'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c',
                     'e', 's', 's', 'o', 'r']
                  : x)))
      (\ x ->
        catcha
          (checkDPProof (append i ['.', '1']) lP lR (computeRuleMap lR)
            (computeRuleMap (reverse_trs lR)) prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 's', 'e', 'm', 'a', 'n', 't',
                   'i', 'c', ' ', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ',
                   'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));
checkDPProof i p r rm rrm (SpscProc sp rP prf) =
  debug i ['S', 'p', 's', 'c', 'P', 'r', 'o', 'c']
    (bind (catcha (create_spsc_proc sp p rP r)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h',
                     'e', ' ', 's', 'u', 'b', 't', 'e', 'r', 'm', ' ', 'c', 'r',
                     'i', 't', 'e', 'r', 'i', 'o', 'n', ' ', 'p', 'r', 'o', 'c',
                     'e', 's', 's', 'o', 'r']
                  : x)))
      (\ x ->
        catcha (checkDPProof (append i ['.', '1']) rP r rm rrm prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 's', 'u', 'b', 't', 'e', 'r',
                   'm', ' ', 'c', 'r', 'i', 't', 'e', 'r', 'i', 'o', 'n', ' ',
                   'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : indent xa))));

checkTRSProof ::
  forall a b c.
    (Eq a, Sshow a, Eq b, Sshow b, Eq c,
      Sshow c) => [Char] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      TRSProof a b c -> Eithera [[Char]] ();
checkTRSProof i r (DPTrans p prf) =
  debug i ['D', 'P', ' ', 't', 'r', 'a', 'n', 's']
    (bind (catcha (checkDPs r p)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g', ' ', 'f',
                     'r', 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
                  : append (showTRS r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'i', 'n', 'i', 't',
                         'i', 'a', 'l', ' ', 'D', 'P', ' ', 'p', 'r', 'o', 'b',
                         'l', 'e', 'm']
                        : append (showDPP p (computeID r)) x))))
      (\ x ->
        catcha
          (checkDPProof (append i ['.', '1']) (mapa (map_rule init_lab) p)
            (mapa (map_rule init_lab) (computeID r))
            (computeRuleMap (mapa (map_rule init_lab) (computeID r)))
            (computeRuleMap
              (reverse_trs (mapa (map_rule init_lab) (computeID r))))
            prf)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 't', 'o', ' ',
                   'd', 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c', 'y', ' ', 'p',
                   'a', 'i', 'r', 's']
                : indent xa))));
checkTRSProof i r (RuleRemoval rp rR prfa) =
  debug i ['r', 'u', 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l']
    (bind (catcha (createRuleRemovalProc rp r rR)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'r',
                     'u', 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l', ' ',
                     'w', 'h', 'e', 'n', ' ', 's', 'w', 'i', 't', 'c', 'h', 'i',
                     'n', 'g', ' ', 'f', 'r', 'o', 'm', ' ', 't', 'h', 'e', ' ',
                     'T', 'R', 'S']
                  : append (showTRS r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S'] :
                        append (showTRS rR) x))))
      (\ x ->
        catcha (checkTRSProof (append i ['.', '1']) rR prfa)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' ', 'r',
                   'e', 'm', 'o', 'v', 'a', 'l', ' ', 'p', 'r', 'o', 'c', 'e',
                   's', 's', 'o', 'r']
                : indent xa))));
checkTRSProof i r (Unlab uR prfa) =
  debug i ['u', 'n', 'l', 'a', 'b', 'e', 'l']
    (bind (catcha (check_unlab_trs r uR)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'u',
                     'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'o', 'f',
                     ' ', 'T', 'R', 'S', ' ', 'w', 'h', 'e', 'n', ' ', 's', 'w',
                     'i', 't', 'c', 'h', 'i', 'n', 'g', ' ', 'f', 'r', 'o', 'm',
                     ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
                  : append (showTRS r)
                      (['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S'] :
                        append (showTRS uR) x))))
      (\ x ->
        catcha (checkTRSProof (append i ['.', '1']) uR prfa)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a', 'b', 'e',
                   'l', 'i', 'n', 'g']
                : indent xa))));
checkTRSProof i r RisEmpty =
  debug i ['R', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (checkB (nulla r)
      (append
        [append i
           [':', ' ', 'R', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
             'p', 't', 'y', ',', ' ', 'b', 'u', 't', ' ', 'i', 't', ' ', 'i',
             's', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']]
        (showTRS r)));
checkTRSProof i r (Semlab sli l lR prfa) =
  debug i ['S', 'e', 'm', 'l', 'a', 'b']
    (bind (catcha (create_sem_lab_trs sli l r lR)
            (\ x ->
              errora
                (append i
                   [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n',
                     ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 's', 'e',
                     'm', 'a', 'n', 't', 'i', 'c', ' ', 'l', 'a', 'b', 'e', 'l',
                     'i', 'n', 'g']
                  : x)))
      (\ x ->
        catcha (checkTRSProof (append i ['.', '1']) lR prfa)
          (\ xa ->
            errora
              (append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 's', 'e', 'm', 'a', 'n', 't', 'i', 'c', ' ', 'l',
                   'a', 'b', 'e', 'l', 'i', 'n', 'g']
                : indent xa))));

}
