(*
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory SubtermCriterionImpl imports SubtermCriterion DpTransImpl begin

datatype 'f spL = SimpProj "('f \<times> nat)list"

definition check_supt ::
  "'f sp \<Rightarrow> ('f::sshow,'v::sshow)rule \<Rightarrow> sstring list monad"
where "check_supt p lr \<equiv> checkB(checkSupterm (lift2term p (fst lr)) (lift2term p (snd lr)))
         [''the projected rhs is not a proper subterm of the projected lhs''] 
      \<leftarrow> (\<lambda>s. (sConcat[''Could not orient rule '',sshow(fst lr),'' -> '',sshow(snd lr)])
           # (sConcat[''i.e., '',sshow(lift2term p (fst lr)), '' |> '',sshow(lift2term p (snd lr))]) # s)
"

definition check_supteq ::
  "'f sp \<Rightarrow> ('f::sshow,'v::sshow)rule \<Rightarrow> sstring list monad"
where "check_supteq p lr \<equiv> checkB(lift2term p (fst lr) = lift2term p (snd lr)
       \<or> checkSupterm (lift2term p (fst lr)) (lift2term p (snd lr)))
         [''the projected rhs is not a subterm of the projected lhs'']
      \<leftarrow> (\<lambda>s. (sConcat [''Could not orient rule '',sshow(fst lr),'' -> '',sshow(snd lr)])
           # (sConcat[''i.e., '',sshow(lift2term p (fst lr)), '' |>= '',sshow(lift2term p (snd lr))]) # s)
"

fun check_projection :: "'f list \<Rightarrow> 'f sp \<Rightarrow> ('f::sshow,'v::sshow)rule \<Rightarrow> sstring list monad"
where "check_projection ds p (Fun f ss,Fun g ts) = (
  checkB(0 < length ss)
  [''lhs is a constant'']
\<guillemotright> checkB(\<not>(f mem ds))
  [sConcat[sshow f,'' is defined in R'']]
\<guillemotright> checkB(p f < length ss)
  [sConcat[sshow(p f),'' is not an argument position of '',sshow(Fun f ss)]]

\<guillemotright>  checkB(0 < length ts)
  [''rhs is a constant'']
\<guillemotright> checkB(\<not>(g mem ds))
  [sConcat[sshow g,'' is defined in R'']]
\<guillemotright> checkB(p g < length ts)
  [sConcat[sshow(p g),'' is not an argument position of '',sshow(Fun g ts)]]
)"
    | "check_projection ds p (s,t) = error [''cannot project a variable'']"

definition check_for_var :: "('f::sshow,'v::sshow)rule \<Rightarrow> sstring list monad"
where "check_for_var rule \<equiv> checkB(\<not> isVar(fst rule))
       [''the lhs is a variable'']
    \<leftarrow> (\<lambda>s. (sConcat[sshow(fst rule),'' is a variable'']) # s)
"

lemma check_supt_sound[simp]:
  assumes "isOK(check_supt p r)"
  shows "(lift2term p (fst r),lift2term p (snd r)) \<in> supt"
proof -
  from assms have "checkSupterm (lift2term p (fst r)) (lift2term p (snd r))"
    unfolding check_supt_def catchSound3 checkBSound .
  thus ?thesis unfolding checkSuptermSound[to_set] by simp
qed

lemma check_supteq_sound[simp]:
  assumes "isOK(check_supteq p r)"
  shows "(lift2term p (fst r),lift2term p (snd r)) \<in> supteq" (is "(?s,?t) \<in> _")
proof -
  from assms have "?s = ?t \<or> checkSupterm ?s ?t"
    unfolding check_supteq_def catchSound3 checkBSound .
  thus ?thesis unfolding checkSuptermSound[to_set] by (rule disjE) auto
qed

lemma check_projection_sound[simp]:
  assumes "isOK(check_projection (computeDSymbs R) p rule)"
  shows "(\<exists>f ss. fst rule = Fun f ss \<and> 0 < length ss \<and> f \<notin> DefSymbs(set R) \<and> p f < length ss)
       \<and> (\<exists>g ts. snd rule = Fun g ts \<and> 0 < length ts \<and> g \<notin> DefSymbs(set R) \<and> p g < length ts)"
proof -
  show ?thesis
  proof (cases rule)
    case (Pair l r)
    show ?thesis
    proof (cases l)
      case (Var x) from assms show ?thesis unfolding Pair Var by simp
    next
      case (Fun f ss) 
      hence Fun1: "l = Fun f ss" .
      show ?thesis
      proof (cases r)
        case (Var y) from assms show ?thesis unfolding Pair Var by simp
      next
        case (Fun g ts)
        with assms show ?thesis unfolding Pair Fun1 Fun by (auto simp: computeDSymbsSound DefSymbs_def)
      qed
    qed
  qed
qed

lemma check_for_var_sound[simp]:
  assumes "isOK(check_for_var rule)"
  shows "\<forall>x. fst rule \<noteq> Var x"
proof -
  show ?thesis
  proof (cases rule)
    case (Pair l r)
    with assms show ?thesis by (cases l) (auto simp: Pair check_for_var_def)
  qed
qed

(*
fun check_spsc_proc ::
  "'f sp \<Rightarrow> sstring list \<Rightarrow>
    ('f::sshow,'v::sshow)trsL \<Rightarrow>
    ('f,'v)trsL \<Rightarrow> ('f,'v)trsL  \<Rightarrow> sstring list monad"
where "check_spsc_proc proj rep P rP R = (
  (let ds = computeDSymbs R in (
    checkAllM (check_projection ds proj) P
    \<leftarrow> (\<lambda>s. ''the supplied projection is malformed'' # s))
  )
\<guillemotright> (let (Sp::('f,'v)trsL,NSp) = computeSplit (\<lambda>r. isOK(check_supt proj r)) P in
  (
    checkAllM (check_supteq proj) NSp
    \<leftarrow> (\<lambda>s. ''problem when orienting DPs'' # s)
  \<guillemotright> checkAllM check_for_var R
    \<leftarrow> (\<lambda>s. ''one of the lhss of R is a variable'' # s)
  \<guillemotright> checkSubset NSp rP
    \<leftarrow> (\<lambda>s. case check_supt proj s of Left err \<Rightarrow> err | Right ok \<Rightarrow> [''internal error'',''please contact one of the CeTA members'',''http://cl-informatik.uibk.ac.at/software/ceta'',''ceta@informatik.uibk.ac.at''])
  ))
\<leftarrow> (\<lambda> s. (''could not apply the subterm criterion processor with the following simple projection''
  # (rep@([]#s))))
)"
*)

fun check_spsc_proc ::
  "'f sp \<Rightarrow> sstring list \<Rightarrow>
    ('f::sshow,'v::sshow)trsL \<Rightarrow>
    ('f,'v)trsL \<Rightarrow> ('f,'v)trsL  \<Rightarrow> sstring list monad"
where "check_spsc_proc proj rep P rP R = (
let ds = computeDSymbs R in (
  let (Sp::('f,'v)trsL,NSp) = computeSplit (\<lambda>r. isOK(check_supt proj r)) P in (
    checkAllM (check_projection ds proj) P
    \<leftarrow> (\<lambda>s. ''the supplied projection is malformed'' # s)
  \<guillemotright> checkAllM (check_supteq proj) NSp
    \<leftarrow> (\<lambda>s. ''problem when orienting DPs'' # s)
  \<guillemotright> checkAllM check_for_var R
    \<leftarrow> (\<lambda>s. ''one of the lhss of R is a variable'' # s)
  \<guillemotright> checkSubset NSp rP
    \<leftarrow> (\<lambda>s. case check_supt proj s of Left err \<Rightarrow> err | Right ok \<Rightarrow> [''internal error'',''please contact one of the CeTA members'',''http://cl-informatik.uibk.ac.at/software/ceta'',''ceta@informatik.uibk.ac.at''])
)
\<leftarrow> (\<lambda> s. (''could not apply the subterm criterion processor with the following simple projection''
  # (rep@([]#s))))
))"

lemma check_spsc_proc_sound[simp] :
  shows "chain_identifying_onlyDP_proc_impl (\<lambda> P R rm rP. isOK(check_spsc_proc proj rep P rP R))"
unfolding chain_identifying_onlyDP_proc_impl_def
proof ((rule allI)+, rule impI)
  fix P R rP
  let ?\<pi> = "lift2term proj"
  assume check: "isOK(check_spsc_proc proj rep P rP R)"
  have "subterm_criterion_proc proj (set P) (set R) (set rP) (set R)"
  proof (cases "computeSplit (\<lambda>dp. isOK(check_supt proj dp)) P")
    case (Pair Sp NSp)
    from check have "\<forall>lr\<in>set R. \<forall>x. fst lr \<noteq> Var x" by auto
    moreover have "spscCond proj (set P) (set rP) (set R)"
    proof -
      from Pair have P: "set P = set NSp \<union> set Sp" by auto
      from check Pair have subset: "set NSp \<subseteq> set rP" by auto
      with P have Sp: "set P - set rP \<subseteq> set Sp" by auto

      from check Pair have "\<forall>lr\<in>set NSp. isOK(check_supteq proj lr)" by (auto simp: Let_def mem_iff)
      with check_supteq_sound[where p = proj]
        have nonstrict: "\<forall>lr\<in>set NSp. (?\<pi>(fst lr),?\<pi>(snd lr)) \<in> supteq" by best

      from check Pair have strict: "\<forall>lr\<in>set Sp. isOK(check_supt proj lr)" by (auto simp: Let_def mem_iff)
      
      with check_supt_sound[where p = proj]
        have "\<forall>lr\<in>set Sp. (?\<pi>(fst lr),?\<pi>(snd lr)) \<in> supt" by best
      hence "\<forall>lr\<in>set Sp. (?\<pi>(fst lr),?\<pi>(snd lr)) \<in> supteq" unfolding supt_iff_supteq_neq[to_set] by auto
      with P nonstrict
        have supteq: "\<forall>lr\<in>set P. (?\<pi>(fst lr),?\<pi>(snd lr)) \<in> supteq"
        (is "\<forall>lr\<in>set P. ?Psupteq (fst lr) (snd lr)") by best

      from Sp strict have "\<forall>lr\<in>set P - set rP. isOK(check_supt proj lr)" by auto
      with check_supt_sound[where p = proj]
        have "\<forall>lr\<in>set P - set rP. (?\<pi>(fst lr),?\<pi>(snd lr)) \<in> supt" by best
      hence A: "\<forall>(l,r)\<in>set P - set rP. (?\<pi> l,?\<pi> r) \<in> supt" unfolding split_def by auto

      from check have "\<forall>lr\<in>set P. isOK(check_projection (computeDSymbs R) proj lr)" by auto
      hence "\<forall>lr\<in>set P. (\<exists>f ss. fst lr = Fun f ss \<and> 0 < length ss \<and> f \<notin> DefSymbs(set R) \<and> proj f < length ss)
                      \<and>  (\<exists>g ts. snd lr = Fun g ts \<and> 0 < length ts \<and> g \<notin> DefSymbs(set R) \<and> proj g < length ts)"
        (is "\<forall>lr\<in>set P. ?P (fst lr) (snd lr)")
        using check_projection_sound[where R = R and p = proj] by blast
      with supteq have "\<forall>lr\<in>set P. ?Psupteq (fst lr) (snd lr) \<and> ?P (fst lr) (snd lr)" by auto
      hence B: "\<forall>(l,r)\<in>set P. ?Psupteq l r \<and> ?P l r" unfolding split_def by auto
      from A and B show ?thesis unfolding spscCond_def by best

    qed
    moreover have "set R = set R" by simp
    ultimately show ?thesis unfolding subterm_criterion_proc_def by auto
  qed
  with ci_subterm_criterion_proc
    show "\<forall>s t \<sigma>. min_ichain (set P,set R) s t \<sigma> \<longrightarrow> (\<exists>i. ichain (set rP,set R) (shift s i) (shift t i) (shift \<sigma> i))"
    unfolding chain_identifying_proc_def by best
qed

fun create_sp :: "'f spL \<Rightarrow> 'f sp"
where "create_sp (SimpProj sp) f = (case lookup f sp of None \<Rightarrow> 0 | Some n \<Rightarrow> n)"

function create_sp_representation :: "('f::sshow)spL \<Rightarrow> sstring list"
where "create_sp_representation(SimpProj []) = []"
    | "create_sp_representation(SimpProj((f,n)#sp)) =
          sConcat[''pi('',sshow f,'') = '',sshow n] # create_sp_representation(SimpProj sp)"
by (pat_completeness, auto)
termination "create_sp_representation"
by (relation "measure (\<lambda>x. case x of SimpProj xs \<Rightarrow> length xs)") auto
  
fun create_spsc_proc :: "'f spL \<Rightarrow> ('f::sshow,'v::sshow)trsL \<Rightarrow> ('f,'v)trsL \<Rightarrow> ('f, 'v)trsL \<Rightarrow> sstring list monad"
where "create_spsc_proc sp P rP R = (let \<pi> = create_sp sp in
  check_spsc_proc \<pi> (create_sp_representation sp) P rP R)"

lemma spsc_proc_ci:
  "chain_identifying_onlyDP_proc_impl(\<lambda>P R rm rP. isOK(create_spsc_proc sp P rP R))"
proof -
  let ?sp = "create_sp sp"
  have simpme: "create_spsc_proc sp = check_spsc_proc ?sp (create_sp_representation sp)" by (rule ext, rule ext, rule ext, simp add: Let_def)
  show ?thesis by (simp only: simpme, rule check_spsc_proc_sound)
qed

declare create_spsc_proc.simps[simp del]

end
